import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A greep made by me.
 * 
 * @author Ninto.
 * @version 0.2
 */
public class Greep extends superClass
{
    Ship homeShip;
    private final int turn = (int)((Greenfoot.getRandomNumber(2)-.5)*20);
    boolean isDead = false;
    boolean isDone = false;
    boolean returned = false;
    
    protected int nextX, nextY;
    protected boolean isCarringFood = false;
    protected int health = 100;
    protected int foodType;
    private int type;
    
    private static GreenfootImage Greep[][] = new GreenfootImage[4][3];
    
    /**
     * Creates a Greep with a homeShip. the Greep will return to this Ship when its called.
     */
    public Greep(Ship myShip) 
    {
        homeShip = myShip;
        setRotation(Greenfoot.getRandomNumber(360));
        if(Greep[0][0] == null) {
            Greep[0][0] = new GreenfootImage("greep.png");
            Greep[0][1] = new GreenfootImage("greep-with-food.png");
            Greep[0][2] = new GreenfootImage("greep-with-pumpkin.png");
            Greep[1][0] = new GreenfootImage("combatGreep.png");
            Greep[1][1] = new GreenfootImage("combatGreep.png");
            Greep[1][2] = new GreenfootImage("combatGreep.png");
            Greep[2][0] = new GreenfootImage("speedGreep.png");
            Greep[2][1] = new GreenfootImage("speedGreep-with-food.png");
            Greep[2][2] = new GreenfootImage("speedGreep-dashing.png");
            Greep[3][0] = new GreenfootImage("workerGreep.png");
            Greep[3][1] = new GreenfootImage("workerGreep.png");
            Greep[3][2] = new GreenfootImage("workerGreep.png");
        }
        if(this instanceof Greep) {setImage(Greep[0][0]); type = 0;}
        if(this instanceof combatGreep) {setImage(Greep[1][0]); type = 1;}
        if(this instanceof speedGreep) {setImage(Greep[2][0]); type = 2;}
        if(this instanceof workerGreep) {setImage(Greep[3][0]); type = 3;}
    }
    
    /**
     * Act - do whatever the Greep wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        if(health <= 0) {
            die();
            return;
        }
        if(homeShip != null && homeShip.isDone) isDone = true;
        deliverFood();
        if(myFood() != null && !isCarringFood) pickUpFood();
        if(isDone) {
            finish();
            if(!returned) move(1);
            return;
        }
        if(myGreep() != null && myGreep().homeShip != homeShip) {
            hurt(myGreep(), 1);
        }
        if(isCarringFood) {
            if(randomChance(50)) {randomWalk();}
            else {returnToShip();}
        }
        else{randomWalk(); checkToSteal();}
        if(!canMoveTo(nextX, nextY)) setRotation(getRotation()+(Greenfoot.getRandomNumber(3)+1)*45);
        else move(1);
    }
    
    /**
     * This method will make the Greep return to its homeShip that's defind in the constructure.
     */
    public void returnToShip()
    {
        if(homeShip == null) {return;}
        setRotation((int)Math.toDegrees(Math.atan2(getY() - homeShip.getY(), getX() - homeShip.getX()))+180);
    }
    
    /**
     * Return true if the Greep can see some food.
     */
    protected Food myFood()
    {
        return (Food)getOneIntersectingObject(Food.class);
    }
    
    /**
     * Return true if the Greep can see an other Greep.
     */
    protected Greep myGreep()
    {
        return (Greep)getOneIntersectingObject(Greep.class);
    }
    
    /**
     * Theres a radom chance that the Greep will rotate.
     */
    protected void randomWalk()
    {
        if(randomChance(50)) {return;}
        if(randomChance(50)) {
            setRotation(getRotation() + 10);
            return;
        }
        else {
            setRotation(getRotation() - 10);
            return;
        }
    }
    
    /**
     * Makes the Greep move in the direction that it's facing.
     */
    protected void move(int speed)
    {
        if(getX() == 0) {setLocation(getWorld().getWidth()-2, getY()); return;}
        if(getX() == getWorld().getWidth()-1) {setLocation(1, getY()); return;}
        if(getY() == 0) {setLocation(getX(), getWorld().getHeight()-2); return;}
        if(getY() == getWorld().getHeight()-1) {setLocation(getX(), 1); return;}
        double angle = Math.toRadians(getRotation());
        int x = (int) Math.round(getX() + Math.cos(angle)*speed);
        int y = (int) Math.round(getY() + Math.sin(angle)*speed);
        nextX = x;
        nextY = y;
        if(canMoveTo(x, y)) setLocation(x, y);
        else{setRotation(getRotation()+turn); move(speed);}
    }
    
    /**
     * The Greeps call this method when there had found some food.
     */
    protected void pickUpFood()
    {
        isCarringFood = true;
        if(myFood().getClass() == Pumpkin.class) {
            ((Pumpkin)myFood()).piecesLeft--;
            setImage(Greep[type][2]);
            foodType = 1;
            return;
        }
        getWorld().removeObject(myFood());
        setImage(Greep[type][1]);
        foodType = 0;
    }
    
    /**
     * When a Greep hit an enemy-Greep will it call this method to reduce the health of the target.
     */
    protected void hurt(Greep myTarget, int myStrength)
    {
        myTarget.health -= myStrength;
    }
    
    protected void checkToSteal()
    {
        if(getOneIntersectingObject(Ship.class) != null && getOneIntersectingObject(Ship.class) != homeShip) {
            ((Ship)getOneIntersectingObject(Ship.class)).stealFood(this);
        }
    }
    
    /**
     * Kills the Greep. (pretty obvious).
     */
    protected void die()
    {
        isDead = true;
        getWorld().addObject(new effect(), getX(), getY());
        if(isCarringFood) {
            switch(foodType) {
                case 0: getWorld().addObject(new Food(), getX(), getY()); break;
                case 1: getWorld().addObject(new Pumpkin(1), getX(), getY()); break;
            }
        }
        getWorld().removeObject(this);
    }
    
    /**
     * Call this when the ship has collected enough food.
     */
    protected void finish()
    {
        if(isDone) {
            if(getOneIntersectingObject(Ship.class) != null && getOneIntersectingObject(Ship.class) == homeShip) {
                isDead = true;
                getWorld().removeObject(this);
                returned = true;
                return;
            }
            if(randomChance(50)) {randomWalk();}
            else {returnToShip();}
        }
    }
    
    /**
     * Give the food this is carring (if any) to the ship.
     */
    protected void deliverFood()
    {
        if(homeShip == null) {return;}
        if(getOneIntersectingObject(Ship.class) == homeShip && isCarringFood) {
            homeShip.addFood();
            isCarringFood = false;
            setImage(Greep[type][0]);
        }
    }
    
    private boolean canMoveTo(int x, int y) {
        java.util.List wall = getWorld().getObjectsAt(x, y, rock.class);
        return wall.isEmpty();
    }
    
    protected void setImage(int img) {
        setImage(Greep[type][img]);
    }
}