import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;

/**
 * The greeps' home ship
 * 
 * @author Ninto.
 * @version 0.1
 */
public class Ship  extends superClass
{
    private float flySpeed = 0;
    public boolean isDone = false;
    
    private int addDelay;
    private final static int max_addDelay = 50;
    private int GreepsLeft;
    private int max_greeps;
    private int combatGreepsLeft;
    private int max_combatGreeps;
    private int speedGreepsLeft;
    private int max_speedGreeps;
    private int workerGreepsLeft;
    private int max_workerGreeps;
    
    private int food = 0;
    private int foodTarget;
    private Color teamColor;
    
    private Greep[] greeps;
    private combatGreep[] combatgreeps;
    private speedGreep[] speedgreeps;
    private workerGreep[] workergreeps;
    private Counter counter;
    
    /**
     * Creates a spaceShip with a certain number of Greebs.
     */
    public Ship(int numGreeps, int numCGreeps, int numSGreeps, int numWGreeps, int numFood, Color myColor) {
        teamColor = myColor;
        foodTarget = numFood;
        max_greeps = numGreeps;
        GreepsLeft = numGreeps;
        max_combatGreeps = numCGreeps;
        combatGreepsLeft = numCGreeps;
        max_speedGreeps = numSGreeps;
        speedGreepsLeft = numSGreeps;
        max_workerGreeps = numWGreeps;
        workerGreepsLeft = numWGreeps;
        
        greeps = new Greep[max_greeps];
        for(int i=0; i<numGreeps; i++) greeps[i] = new Greep(this);
        
        combatgreeps = new combatGreep[max_combatGreeps];
        for(int i=0; i<numCGreeps; i++) combatgreeps[i] = new combatGreep(this);
        
        speedgreeps = new speedGreep[max_speedGreeps];
        for(int i=0; i<numSGreeps; i++) speedgreeps[i] = new speedGreep(this);
        
        workergreeps = new workerGreep[max_workerGreeps];
        for(int i=0; i<numWGreeps; i++) workergreeps[i] = new workerGreep(this);
        
        counter = new Counter(teamColor, foodTarget);
    }
    
    protected void addedToWorld(World world) {
        if(getOneObjectAtOffset(getX(), getY()+50, Counter.class) != counter)
        getWorld().addObject(counter, getX(), getY()+50);
    }
    
    /**
     * Act - do whatever the Ship wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() {
        if(allGreepsReturned()) {fly(); return;}
        if(!isDone && food >= foodTarget || !isDone && getWorld().finished) {
            isDone = true;
            return;
        }
        addGreeps();
    }
    
    /**
     * The Greeps will call this method when there return with some food.
     */
    public void addFood() {
        food++;
        counter.setValue(food);
    }
    
    /**
     * The anemy Greeps will call this method when there come to steal it.
     */
    public void stealFood(Greep thief) {
        if(food <= 0 || isDone) {return;}
        thief.foodType = 0;
        thief.isCarringFood = true;
        thief.setImage(1);
        food--;
        counter.setValue(food);
    }
    
    /**
     * When the ship has collected enough food will it start to fly using this method.
     */
    private void fly() {
        getWorld().removeObject(counter);
        if(getY() < 1) {getWorld().removeObject(this); return;}
        flySpeed+=0.2f;
        setLocation(getX(), getY() - (int)flySpeed);
    }
    
    /**
     * Adds all the Greeps with a little delay.
     */
    private void addGreeps() {
        if(addDelay > 0) {addDelay--; return;}
        addDelay = max_addDelay;
        if(GreepsLeft > 0) {
            GreepsLeft--;
            getWorld().addObject(greeps[GreepsLeft], getX(), getY());
            return;
        }
        if(combatGreepsLeft > 0) {
            combatGreepsLeft--;
            getWorld().addObject(combatgreeps[combatGreepsLeft], getX(), getY());
            return;
        }
        if(speedGreepsLeft > 0) {
            speedGreepsLeft--;
            getWorld().addObject(speedgreeps[speedGreepsLeft], getX(), getY());
            return;
        }
        if(workerGreepsLeft > 0) {
            workerGreepsLeft--;
            getWorld().addObject(workergreeps[workerGreepsLeft], getX(), getY());
            return;
        }
    }
    
    /**
     * Returns true if all Greeps of this ship has returned.
     */
    private boolean allGreepsReturned() {
        for(int i=0; i<max_greeps; i++) {
            if(!greeps[i].returned && !greeps[i].isDead) return false;
        }
        for(int i=0; i<max_combatGreeps; i++) {
            if(!combatgreeps[i].returned && !combatgreeps[i].isDead) return false;
        }
        for(int i=0; i<max_speedGreeps; i++) {
            if(!speedgreeps[i].returned && !speedgreeps[i].isDead) return false;
        }
        for(int i=0; i<max_workerGreeps; i++) {
            if(!workergreeps[i].returned && !workergreeps[i].isDead) return false;
        }
        return true;
    }
}